
// first argument: path of macro for batch mode
// second argument: physics list name
// ex.) ./bin/Linux-g++/(program name) macros/vis.mac QGSP_INCLXX_XS

#include "G4RunManager.hh"

#include "DetectorConstruction.hh"

#include "G4PhysListFactorySAMURAI.hh"
#include "G4VModularPhysicsList.hh"

#include "PrimaryGeneratorActionBasic.hh"
#include "RunActionBasic.hh"
#include "EventActionBasic.hh"
#include "TDataInitializerInterface.hh"
#include "TDataInitializerNeutronDetector.hh"
#include "TSimulationParameterGlobal.hh"
#include "TTypeAnalyser.hh"

#include "G4VisExecutive.hh"
#include "G4TrajectoryDrawByParticleID.hh"

#include "G4UIExecutive.hh"
#include "G4UImanager.hh"

int main(int argc,char** argv)
{
  // Construct the default run manager
  G4RunManager* runManager = new G4RunManager;

  // set detector construction class
  G4VUserDetectorConstruction* userDetectorConstruction = new DetectorConstruction;
  runManager->SetUserInitialization(userDetectorConstruction);

  // set physics construction class
  G4String physName = "QGSP_INCLXX_XS"; // default
  // Physics List name defined via 3rd argument
  if(argc==3){
    physName = argv[2];
  }
  G4PhysListFactorySAMURAI physListFactorySAMURAI;
  G4VModularPhysicsList* modularPhysicsList = physListFactorySAMURAI.GetReferencePhysList(physName);
  if(!modularPhysicsList) return 1; // exit failure
  runManager->SetUserInitialization(modularPhysicsList);

  // initialize
  runManager->Initialize();

  // set user action classes
  G4VUserPrimaryGeneratorAction* userPrimaryGeneratorAction = new PrimaryGeneratorActionBasic;
  TDataInitializerInterface* dataInitializerInterface = new TDataInitializerNeutronDetector;
  G4UserRunAction* userRunAction = new RunActionBasic(dataInitializerInterface);
  G4UserEventAction* userEventAction = new EventActionBasic(dataInitializerInterface);
  runManager->SetUserAction(userPrimaryGeneratorAction);
  runManager->SetUserAction(userRunAction);
  runManager->SetUserAction(userEventAction);

  // save class name which was used
  if(!gSimulationParameterGlobal){
    gSimulationParameterGlobal = new TSimulationParameterGlobal;
  }
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*userDetectorConstruction));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*modularPhysicsList));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*userPrimaryGeneratorAction));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*dataInitializerInterface));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*userRunAction));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*userEventAction));
  gSimulationParameterGlobal->AddClassName(TTypeAnalyser::TypeName(*userDetectorConstruction));

  G4VisManager* visManager = 0;
  visManager = new G4VisExecutive;
  visManager->Initialize();

  G4TrajectoryDrawByParticleID* trajectoryModel = new G4TrajectoryDrawByParticleID;
  trajectoryModel->Set("proton","red");
  trajectoryModel->Set("antiproton","yellow");
  trajectoryModel->Set("neutron","green");
  trajectoryModel->Set("e-","blue");
  trajectoryModel->Set("e+","cyan");
  trajectoryModel->Set("gamma","white");
  trajectoryModel->Set("mu-","magenta");      
  trajectoryModel->Set("mu+","magenta");      
  trajectoryModel->Set("pi-","black");      
  trajectoryModel->Set("pi+","black");      

  visManager->RegisterModel(trajectoryModel);

  if(argc==1){ // interactive mode
    G4UIExecutive* uiExecutive = new G4UIExecutive(argc, argv);
    uiExecutive->SessionStart();
    delete uiExecutive;
  }else{ // Batch mode
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    G4UImanager* UImanager = G4UImanager::GetUIpointer();
    UImanager->ApplyCommand(command+fileName);
  }

  delete visManager;
  delete runManager;

  return 0;
}
