
// .L DemocraticDecay_b01b12.C+
// DemocraticDecay_b01b12(1)

#include "TLorentzVector.h"
#include "TGenPhaseSpace.h"
#include "TRandom.h"
#include "TH1D.h"
#include "TH2D.h"

#include <iostream>

Int_t fAbeam=14;
Int_t fZbeam=4;
Int_t fAcore=12;
Int_t fZcore=4;
Int_t fNdecay=3;
Double_t mu=931.494;
Double_t mf = fAcore*mu+25.076506;
Double_t mn = mu+8.07131710;
Double_t vlight = 29.979;

Double_t fBeamEnergy=220;

static const int NDECAY_MAX=10;
TGenPhaseSpace* fPhaseSpace=0;
TLorentzVector* fP[NDECAY_MAX];

void InitDemocraticDecay(double erel);

void DemocraticDecay_b01b12(double erel, int number=100000)
{
  InitDemocraticDecay(erel);
  TH1* hb01b12 = new TH1D("beta01/beta12","title",20000,-10,10);

  for(int i=0;i<number;++i){
    double weight;
    weight = fPhaseSpace->Generate();
    while(gRandom->Uniform()>weight) weight = fPhaseSpace->Generate();

    fP[0] = fPhaseSpace->GetDecay(0);
    fP[1] = fPhaseSpace->GetDecay(1);
    fP[2] = fPhaseSpace->GetDecay(2);

    double beta0 = sqrt(1/(mn*mn/(*fP)[0].P()/(*fP)[0].P()+1));
    double gamma0 = 1/sqrt(1-beta0*beta0);
    double betax0 = (*fP)[0].Px()/mn/gamma0;
    double betay0 = (*fP)[0].Py()/mn/gamma0;
    double betaz0 = (*fP)[0].Pz()/mn/gamma0;
    double z0 = 1000; // 10m
    double t0 = z0/(betaz0*vlight);
    double x0 = t0*betax0*vlight;
    double y0 = t0*betay0*vlight;

    double beta1 = sqrt(1/(mn*mn/(*fP)[1].P()/(*fP)[1].P()+1));
    double gamma1 = 1/sqrt(1-beta1*beta1);
    double betax1 = (*fP)[1].Px()/mn/gamma1;
    double betay1 = (*fP)[1].Py()/mn/gamma1;
    double betaz1 = (*fP)[1].Pz()/mn/gamma1;
    double z1 = 1000; // 10m
    double t1 = z1/(betaz1*vlight);
    double x1 = t1*betax1*vlight;
    double y1 = t1*betay1*vlight;

    double beta2 = sqrt(1/(mn*mn/(*fP)[2].P()/(*fP)[2].P()+1));
    double gamma2 = 1/sqrt(1-beta2*beta2);
    double betax2 = (*fP)[2].Px()/mn/gamma2;
    double betay2 = (*fP)[2].Py()/mn/gamma2;
    double betaz2 = (*fP)[2].Pz()/mn/gamma2;
    double z2 = 1100; // 11m
    double t2 = z2/(betaz2*vlight);
    double x2 = t2*betax2*vlight;
    double y2 = t2*betay2*vlight;

    double dx = x2-x1;
    double dy = y2-y1;
    double dz = z2-z1;
    double dr = sqrt(dx*dx+dy*dy+dz*dz);
    double dt = t2-t1;

    double beta12 = dr/dt/vlight;
    double beta01 = (t1<t2) ? beta1 : beta2;

    hb01b12->Fill(beta01/beta12);

    TLorentzVector Ptot;
    double Mtot=0;
    for(int i=0;i<fNdecay;++i){
      Ptot += (*fP)[i];
      Mtot += fP[i]->M();
    }
  }
}

void InitDemocraticDecay(double erel)
{
  double Masses[NDECAY_MAX];// for TGenPhaseSpace
  Double_t Mbeam = 0;
  Masses[0] = mf;
  Masses[1] = mn;
  Masses[2] = mn;
  for(int i=0;i<fNdecay;++i){
    Mbeam += Masses[i];
  }
  Mbeam += erel;
  Double_t Tbeam = fBeamEnergy*fAbeam;// Total Kinetic energy
  Double_t Ebeam = Tbeam + Mbeam;
  Double_t Pbeam_abs = sqrt(Tbeam*(Tbeam+2.0*Mbeam));
  TLorentzVector Pbeam(0.0, 0.0, Pbeam_abs, Ebeam);

  delete fPhaseSpace;
  fPhaseSpace = new TGenPhaseSpace;
  fPhaseSpace->SetDecay(Pbeam,fNdecay,Masses);
}
