
// .x DemocraticDecay_print.C+

#include "TLorentzVector.h"
#include "TGenPhaseSpace.h"
#include "TRandom.h"
#include "TH1D.h"
#include "TH2D.h"

#include <iostream>

Int_t fAbeam=14;
Int_t fZbeam=4;
Int_t fAcore=12;
Int_t fZcore=4;
Double_t fErel=1;
Int_t fNdecay=3;

Double_t mu=931;
Double_t fBeamEnergy=220;

static const int NDECAY_MAX=10;
TGenPhaseSpace* fPhaseSpace=0;
TLorentzVector* fP[NDECAY_MAX];

void InitDemocraticDecay();

void DemocraticDecay_print()
{
  InitDemocraticDecay();

  double weight;
  weight = fPhaseSpace->Generate();
  while(gRandom->Uniform()>weight) weight = fPhaseSpace->Generate();

  for(int i=0;i<fNdecay;++i){
    fP[i] = fPhaseSpace->GetDecay(i);
    std::cout<< "  Mass="<<(*fP)[i].M()<<"MeV"
	     << "  Px="<<(*fP)[i].Px()<<"MeV/c"
	     << "  Py="<<(*fP)[i].Py()<<"MeV/c"
	     << "  Pz="<<(*fP)[i].Pz()<<"MeV/c"
	     << "  P="<<(*fP)[i].P()<<"MeV/c"
	     << std::endl;
  }

  TLorentzVector Ptot;
  double Mtot=0;
  for(int i=0;i<fNdecay;++i){
    Ptot += (*fP)[i];
    Mtot += fP[i]->M();
  }
  std::cout<<"Erel check="<<(Ptot.M()-Mtot)<<"MeV"<<std::endl;
}

void InitDemocraticDecay()
{
  double Masses[NDECAY_MAX];// for TGenPhaseSpace
  Double_t Mbeam = 0;
  Masses[0] = fAcore*mu;
  Masses[1] = mu+8;
  Masses[2] = mu+8;
  for(int i=0;i<fNdecay;++i){
    Mbeam += Masses[i];
  }
  Mbeam += fErel;
  Double_t Tbeam = fBeamEnergy*fAbeam;// Total Kinetic energy
  Double_t Ebeam = Tbeam + Mbeam;
  Double_t Pbeam_abs = sqrt(Tbeam*(Tbeam+2.0*Mbeam));
  TLorentzVector Pbeam(0.0, 0.0, Pbeam_abs, Ebeam);

  delete fPhaseSpace;
  fPhaseSpace = new TGenPhaseSpace;
  fPhaseSpace->SetDecay(Pbeam,fNdecay,Masses);
}
