#ifndef PRIMARYGENERATORACTIONBASIC_HH
#define PRIMARYGENERATORACTIONBASIC_HH

#include "G4VUserPrimaryGeneratorAction.hh"
#include "globals.hh"

class G4ParticleGun;
class G4ParticleDefinition;

class TRandom3;
class TGenPhaseSpace;
class TLorentzVector;

class ActionBasicMessenger;

class PrimaryGeneratorActionBasic : public G4VUserPrimaryGeneratorAction
{
public:
  PrimaryGeneratorActionBasic(G4int seed = 128);
  virtual ~PrimaryGeneratorActionBasic();

  bool SetBeamType(G4String beamType);
  void GeneratePrimaries(G4Event*);

  virtual void AccumulateBeamData(G4Event* anEvent);

  bool SetBeamEnergy(G4double energy);
  bool SetBeamSigma(G4double sigma);
  bool SetBeamTheta(G4double theta);

  void BeamTypeGeantino(G4Event* anEvent);
  void BeamTypeMonoEnergy(G4Event* anEvent);
  void BeamTypeMonoEnergyRandTheta(G4Event* anEvent);
  void BeamTypeMonoEnergyRandMomentum(G4Event* anEvent);
  void BeamTypeMonoEnergyRandMomentumRandPosition(G4Event* anEvent);
  void BeamTypeGausEnergy(G4Event* anEvent);
  void BeamTypeGausEnergyRandTheta(G4Event* anEvent);
  void BeamTypeExpEnergyRandMomentum(G4Event* anEvent);
  void BeamTypeCs137(G4Event* anEvent);
  void BeamTypeCo60(G4Event* anEvent);
  void BeamTypeY88(G4Event* anEvent);
  void BeamTypeBeamInducedGamma(G4Event* anEvent);

  void BeamTypeDemocraticDecay(G4Event* anEvent);
  bool SetErel(G4double erel);
  bool SetAbeam(G4int A){fAbeam=A;return true;}
  bool SetZbeam(G4int Z){fZbeam=Z;return true;}
  bool SetAcore(G4int A){fAcore=A;return true;}
  bool SetZcore(G4int Z){fZcore=Z;return true;}
  bool InitDemocraticDecay();

protected:
  G4ParticleGun* fParticleGun;

  TRandom3* fRandom;
  G4double fBeamEnergy;
  G4double fBeamSigma;
  G4double fBeamTheta;

  G4bool fOKInitDemocraticDecay;
  G4int fAbeam,fZbeam;
  G4int fAcore,fZcore;
  G4double fErel;
  G4int fNdecay;

  static const int NDECAY_MAX=10;
  TGenPhaseSpace* fPhaseSpace;
  TLorentzVector* fP[NDECAY_MAX];
  G4ParticleDefinition *fParticles[NDECAY_MAX];

  void (PrimaryGeneratorActionBasic::*beamFunctionPtr)(G4Event*);

  ActionBasicMessenger* fActionBasicMessenger;
};

#endif


