#include "ActionBasicMessenger.hh"
#include "PrimaryGeneratorActionBasic.hh"
#include "ActionCommonVariable.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"

ActionBasicMessenger* ActionBasicMessenger::fActionBasicMessenger(0);

ActionBasicMessenger* ActionBasicMessenger::GetInstance()
{
  if(fActionBasicMessenger == 0){
    fActionBasicMessenger = new ActionBasicMessenger;
  }
  return fActionBasicMessenger;
}

ActionBasicMessenger::ActionBasicMessenger()
  : fPrimaryGeneratorActionBasic(0)
{
  fActionCommonVariable = ActionCommonVariable::GetInstance();

  fActionDir = new G4UIdirectory("/action/");
  fActionDir->SetGuidance(" Set Action Parameters.");

  fRunNameCmd = new G4UIcmdWithAString("/action/RunName",this);
  fRunNameCmd->SetGuidance("Set run name");
  fRunNameCmd->SetParameterName("runName",false);
  fRunNameCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fOverWriteCmd = new G4UIcmdWithAString("/action/OverWrite",this);
  fOverWriteCmd->SetGuidance("OverWrite ROOT file [y/n/ask]");
  fOverWriteCmd->SetParameterName("overWrite",false);
  fOverWriteCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fUseTreeCmd = new G4UIcmdWithABool("/action/UseTree",this);
  fUseTreeCmd->SetGuidance("Set use Tree or not");
  fUseTreeCmd->SetParameterName("useTree",false);
  fUseTreeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fTreeNameCmd = new G4UIcmdWithAString("/action/TreeName",this);
  fTreeNameCmd->SetGuidance("Set tree name");
  fTreeNameCmd->SetParameterName("treeName",false);
  fTreeNameCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fTreeTitleCmd = new G4UIcmdWithAString("/action/TreeTitle",this);
  fTreeTitleCmd->SetGuidance("Set tree title");
  fTreeTitleCmd->SetParameterName("treeTitle",false);
  fTreeTitleCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamTypeCmd = new G4UIcmdWithAString("/action/BeamType",this);
  fBeamTypeCmd->SetGuidance("Set beam type");
  fBeamTypeCmd->SetParameterName("beamType",false);
  fBeamTypeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamEnergyCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamEnergy",this);
  fBeamEnergyCmd->SetGuidance("Set beam energy");
  fBeamEnergyCmd->SetParameterName("beamEnergy",false);
  fBeamEnergyCmd->SetUnitCategory("Energy");
  fBeamEnergyCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamSigmaCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamSigma",this);
  fBeamSigmaCmd->SetGuidance("Set beam sigma");
  fBeamSigmaCmd->SetParameterName("beamSigma",false);
  fBeamSigmaCmd->SetUnitCategory("Energy");
  fBeamSigmaCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamThetaCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamTheta",this);
  fBeamThetaCmd->SetGuidance("Set beam theta");
  fBeamThetaCmd->SetParameterName("beamTheta",false);
  fBeamThetaCmd->SetUnitCategory("Angle");
  fBeamThetaCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  // for Decay simulation
  fActionDemocraticDecayDir = new G4UIdirectory("/action/DemocraticDecay/");
  fActionDemocraticDecayDir->SetGuidance(" Set DemocraticDecay Parameters.");

  fDecayErelCmd = new G4UIcmdWithADoubleAndUnit("/action/DemocraticDecay/Erel",this);
  fDecayErelCmd->SetGuidance("Set Erel for Decay");
  fDecayErelCmd->SetParameterName("Erel",false);
  fDecayErelCmd->SetUnitCategory("Energy");
  fDecayErelCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fDecayAcoreCmd = new G4UIcmdWithAnInteger("/action/DemocraticDecay/Acore",this);
  fDecayAcoreCmd->SetGuidance("Define core A");
  fDecayAcoreCmd->SetParameterName("Acore",false);
  fDecayAcoreCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fDecayZcoreCmd = new G4UIcmdWithAnInteger("/action/DemocraticDecay/Zcore",this);
  fDecayZcoreCmd->SetGuidance("Define core Z");
  fDecayZcoreCmd->SetParameterName("Zcore",false);
  fDecayZcoreCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fDecayAbeamCmd = new G4UIcmdWithAnInteger("/action/DemocraticDecay/Abeam",this);
  fDecayAbeamCmd->SetGuidance("Define beam A");
  fDecayAbeamCmd->SetParameterName("Abeam",false);
  fDecayAbeamCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fDecayZbeamCmd = new G4UIcmdWithAnInteger("/action/DemocraticDecay/Zbeam",this);
  fDecayZbeamCmd->SetGuidance("Define beam Z");
  fDecayZbeamCmd->SetParameterName("Zbeam",false);
  fDecayZbeamCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

ActionBasicMessenger::~ActionBasicMessenger()
{
  delete fRunNameCmd;
  delete fOverWriteCmd;
  delete fUseTreeCmd;
  delete fTreeNameCmd;
  delete fTreeTitleCmd;
  delete fBeamTypeCmd;
  delete fBeamEnergyCmd;
  delete fBeamSigmaCmd;
  delete fBeamThetaCmd;
  delete fActionDir;

  delete fDecayErelCmd;
  delete fDecayAbeamCmd;
  delete fDecayZbeamCmd;
  delete fDecayAcoreCmd;
  delete fDecayZcoreCmd;
  delete fActionDemocraticDecayDir;

  fActionBasicMessenger = 0;
}

void ActionBasicMessenger::Register(PrimaryGeneratorActionBasic* primaryGeneratorActionBasic)
{
  fPrimaryGeneratorActionBasic = primaryGeneratorActionBasic;
}

void ActionBasicMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if(command == fRunNameCmd){
    fActionCommonVariable->SetRunName(newValue);
  }else if(command == fOverWriteCmd){
    fActionCommonVariable->SetOverWrite(newValue);
  }else if(command == fUseTreeCmd){
    fActionCommonVariable->SetUseTree(fUseTreeCmd->GetNewBoolValue(newValue));
  }else if(command == fTreeNameCmd){
    fActionCommonVariable->SetTreeName(newValue);
  }else if(command == fTreeTitleCmd){
    fActionCommonVariable->SetTreeTitle(newValue);
  }else if(command == fBeamTypeCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamType(newValue)) G4Exception(0, 0, FatalException, 0);
  }else if(command == fBeamEnergyCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamEnergy(fBeamEnergyCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fBeamSigmaCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamSigma(fBeamSigmaCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fBeamThetaCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamTheta(fBeamThetaCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fDecayErelCmd){
    if(!fPrimaryGeneratorActionBasic->SetErel(fDecayErelCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fDecayAbeamCmd){
    if(!fPrimaryGeneratorActionBasic->SetAbeam(fDecayAbeamCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fDecayZbeamCmd){
    if(!fPrimaryGeneratorActionBasic->SetZbeam(fDecayZbeamCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fDecayAcoreCmd){
    if(!fPrimaryGeneratorActionBasic->SetAcore(fDecayAcoreCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fDecayZcoreCmd){
    if(!fPrimaryGeneratorActionBasic->SetZcore(fDecayZcoreCmd->GetNewIntValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }
}

