
#include "TAnalysisInterface.hh"

#include "TFile.h"

#include <iostream>
#include <string>

ClassImp(TAnalysisInterface)

TAnalysisInterface::TAnalysisInterface()
: fOutputFileName("temp.root"), fNumberOfEntries(-1), fInitial(1)
{;}

TAnalysisInterface::~TAnalysisInterface()
{}

int TAnalysisInterface::book(std::string rootfile)
{
  fROOTFileNameArray.push_back(rootfile);

  return 1;
}

int TAnalysisInterface::SetOutputFileName(std::string rootfile)
{
  fOutputFileName = rootfile;

  return 1;
}

int TAnalysisInterface::StartAnalysis(Long64_t numberOfEntries)
{
  fNumberOfEntries = numberOfEntries;

  fFileExp = TFile::Open(fOutputFileName.c_str(),"recreate");
  if(!fFileExp){
    std::cerr << " cannot open \"" << fOutputFileName << "\"" << std::endl;
    return 0;
  }

  while(!fROOTFileNameArray.empty()){
    std::string rootFileName = fROOTFileNameArray.front();
    fROOTFileNameArray.pop_front();

    std::cout << " analysis " << rootFileName << std::endl;
    fFileSim = TFile::Open(rootFileName.c_str(), "READ");
    if(!fFileSim){
      std::cerr << " cannot open \"" << rootFileName << "\"" << std::endl;
      continue;
    }

    if(!ReadParameters(fFileSim)){
      fFileSim->Close();
      continue;
    }
    
    PrintParameters();

    // do only at first time
    if(fInitial){
      fFileExp->cd();
      CreateHist();
      fInitial = 0;
    }

    if(!CreateAndAnalyzeExpData()) break;

    fFileSim->Close();
  }

  fFileExp->Write();
  fFileExp->cd();

  return 1;
}

int TAnalysisInterface::CreateAndAnalyzeExpData()
{
  std::cerr << "TAnalysisInterface::CreateAndAnalyzeExpData is not yet implemented generaly. Please implement by each derived class."
	    << std::endl;

  return 0;
}
