
#include "BeamPipeConstruction.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

BeamPipeConstruction::BeamPipeConstruction()
  : fLogicBeamPipe(0), fLogicWorld(0)
{
  fBeamPipeMotherMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fBeamPipeMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Al");
  fPbShieldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Pb");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

BeamPipeConstruction::~BeamPipeConstruction()
{;}

G4VPhysicalVolume* BeamPipeConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct BeamPipe LV
  fLogicBeamPipe = ConstructSub();

  // BeamPipe position
  //  G4ThreeVector beamPipePos(0, 0, 176*mm);
  G4ThreeVector beamPipePos(0, 0, 0*mm);

  // World LV
  G4ThreeVector worldSize(2*m,2*m,2*m);
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // beamPipe PV
  new G4PVPlacement(0,
		    beamPipePos,
		    fLogicBeamPipe,
		    "BeamPipe",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNebula->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* BeamPipeConstruction::ConstructSub()
{
  G4cout << "Creating Beam Pipe" << G4endl;

  G4Tubs* sBeamPipeMother = new G4Tubs("sBeamPipeMother",144*mm/2,151*mm/2,1184.0*mm/2,0*deg,360*deg);
  G4LogicalVolume* lBeamPipeMother = new G4LogicalVolume(sBeamPipeMother,fBeamPipeMotherMaterial,"lBeamPipeMother",0,0,0);
  lBeamPipeMother->SetVisAttributes(new G4VisAttributes(G4Colour(1,1,0)));

  G4Tubs* sBeamPipe = new G4Tubs("sBeamPipe",144*mm/2,150*mm/2,1184.0*mm/2,0*deg,360*deg);
  G4LogicalVolume* lBeamPipe = new G4LogicalVolume(sBeamPipe,fBeamPipeMaterial,"lBeamPipe",0,0,0);
  lBeamPipe->SetVisAttributes(new G4VisAttributes(G4Colour(1,1,1)));

  G4Tubs* sPbShield = new G4Tubs("sPbShield",150*mm/2,151*mm/2,1184.0*mm/2,0*deg,360*deg);
  G4LogicalVolume* lPbShield = new G4LogicalVolume(sPbShield,fPbShieldMaterial,"lPbShield",0,0,0);
  lPbShield->SetVisAttributes(new G4VisAttributes(G4Colour(0,1,1)));

  new G4PVPlacement(0,G4ThreeVector(0,0,0),lBeamPipe,"pBeamPipe",lBeamPipeMother,false, 0);
  new G4PVPlacement(0,G4ThreeVector(0,0,0),lPbShield,"pPbShield",lBeamPipeMother,false, 0);
  
  return lBeamPipeMother;
}
