
#include "SourceConstruction.hh"

#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

SourceConstruction::SourceConstruction()
  : fLogicSource(0), fLogicWorld(0)
{
  fSourceMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_C");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

SourceConstruction::~SourceConstruction()
{;}

G4VPhysicalVolume* SourceConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct Source LV
  fLogicSource = ConstructSub();

  // Source position
  G4ThreeVector sourcePos(0, 0, 0);

  // World LV
  G4ThreeVector worldSize(2*fDiameter,
			  2*fDiameter,
			  2*fThickness);
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // source PV
  new G4PVPlacement(0,
		    sourcePos,
		    fLogicSource,
		    "Source",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNebula->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* SourceConstruction::ConstructSub()
{
  std::cout << "Createing Source" << std::endl;

  fThickness = 11.27*mm;
  fDiameter = 40.*mm;
  G4Tubs* solidSource = new G4Tubs("solidSource",0.0*mm,fDiameter/2,fThickness/2,0*deg,360*deg);
  G4LogicalVolume* logicSource = new G4LogicalVolume(solidSource,fSourceMaterial,"Source");

  logicSource->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));

  return logicSource;
}
