
#include "TargetConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

TargetConstruction::TargetConstruction()
  : fLogicTarget(0), fLogicWorld(0)
{
  fTargetMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Li");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

TargetConstruction::~TargetConstruction()
{;}

G4VPhysicalVolume* TargetConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct Target LV
  fLogicTarget = ConstructSub();

  // Target position
  G4ThreeVector targetPos(0, 0, 0);

  // World LV
  G4ThreeVector worldSize(2*fTargetSize.x(),
			  2*fTargetSize.y(),
			  2*fTargetSize.z());
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // target PV
  new G4PVPlacement(0,
		    targetPos,
		    fLogicTarget,
		    "Target",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNebula->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* TargetConstruction::ConstructSub()
{
  std::cout << "Createing Target" << std::endl;

  // target LV
  G4ThreeVector targetSize(4*cm, 4*cm, 2.04*cm);
  fTargetSize = targetSize;

  G4Box* solidTarget = new G4Box("Target",
				 targetSize.x()/2,
				 targetSize.y()/2,
				 targetSize.z()/2);
  G4LogicalVolume* logicTarget = new G4LogicalVolume(solidTarget,fTargetMaterial,"Target");

  logicTarget->SetVisAttributes(new G4VisAttributes(G4Colour::Red()));

  return logicTarget;
}
